/*
 * GFCLinearizer.java
 * Copyright (C) 2004-2005, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.linearize;

import se.chalmers.cs.gf.GFException;
import se.chalmers.cs.gf.abssyn.*;
import se.chalmers.cs.gf.gfcutil.*;
import se.chalmers.cs.gf.linearize.gfvalue.*;
import se.chalmers.cs.gf.GFC.*;
import se.chalmers.cs.gf.GFC.Absyn.*;
import static se.chalmers.cs.gf.linearize.LinearizeLogger.*;

import java.util.*;
import java.io.Serializable;

/**
 *  A linearizer which interprets a concrete GFC module.
 */
public class GFCLinearizer implements Serializable,Linearizer {

        private GFCModule module;

        /**
         *  @param module A GFC module
         */
        public GFCLinearizer(GFCModule module) {
                this.module = module;
        }

        public Value linearize(Tree t) {
                log.fine("Linearizing with " + getName());
                Value v = t.accept(new TreeLinearizer(), this);
                log.finer("Done: " + v);
                return v;
        }

        public String getName() {
                return module.getName();
        }

        public boolean tryThisOne() {
                return true;
        }

        /**
         *  Linearizes an abstract syntax tree by recursively applying 
         *  the linearization rules for each function.
         */
        private static class TreeLinearizer 
                implements TreeVisitor<Value,GFCLinearizer> {

                public Value visit(Fun t, GFCLinearizer rules) {
                        Tree[] c = t.getChildren();
                        Value[] args = new Value[c.length];
                        for (int i = 0; i < c.length; i++)
                                args[i] = c[i].accept(this, rules);
                        String label = t.getLabel();
                        LinRule rule = rules.module.findLinRule(label);
                        // FIXME: hacky, needed to work around missing
                        // linearizations for type arguments
                        if (rule == null)
                                return Record.ss("<missing lin:" + label + ">");

                        if (rule.getArity() != args.length)
                                throw new GFException(label + " requires " + rule.getArity() 
                                                           + " arguments, got " + args.length);
                        return rule.getTerm().accept(new InterpretLinRule(), args);
                }

                public Value visit(IntLiteral l, GFCLinearizer rules) {
                        return Record.ss(String.valueOf(l.getValue()));
                }

                public Value visit(StringLiteral l, GFCLinearizer rules) {
                        return Record.ss(l.getValue());
                }

                public Value visit(MetaVariable v, GFCLinearizer rules) {
                        // FIXME: use lindef
                        return new Meta();
                        //return Record.ss("? : " + v.getType());
                }

        }

}
